@@ -1,6 +1,6 @@
 from typing import Union
-from src import Data
-from src import DGModel
+from simcalibration.utils.Data import Data
+from simcalibration.dg_models.DGModel import DGModel
 from pgmpy.models import BayesianModel
 from pgmpy.estimators import *
 
