@@ -1,6 +1,6 @@
-from src.simcalibration.dg_models.DGModel import DGModel
+from simcalibration.dg_models.DGModel import DGModel
 from pomegranate.BayesianNetwork import BayesianNetwork
-from src.simcalibration.utils.Data import Data
+from simcalibration.utils.Data import Data
 from abc import ABC, abstractmethod
 
 
