@@ -7,16 +7,14 @@ import scipy
 from matplotlib import transforms
 from matplotlib.pyplot import scatter
 from numpy import mean, int64
-from transpose_dict import TD
 from scipy.stats import sem, kendalltau, rankdata, stats
 import numpy as np
 import seaborn as sns
-figuredirname = os.sep+"figures"+os.sep
 
 class Postprocessing():
     def __init__(self):
-        if os.path.isdir(os.getcwd() + figuredirname) == False:
-            os.mkdir(os.getcwd() + figuredirname)
+        self.figuredirname = os.path.join(os.getcwd(), "results")
+        os.makedirs(self.figuredirname, exist_ok=True)
         pass
 
     def realworld_benchmarks_visualise(self, analysis0_results: pd.DataFrame):
@@ -40,7 +38,7 @@ class Postprocessing():
 
             # Save the figure and show
             plt.tight_layout()
-            plt.savefig(os.getcwd()+figuredirname+'realworld_benchmarks_bar_plot_with_error_bars.png')
+            plt.savefig(os.path.join(self.figuredirname, 'realworld_benchmarks_bar_plot_with_error_bars.png'))
             plt.show()
 
     def realworld_benchmarks_bootstrapping_visualise(self, analysis1_results: pd.DataFrame):
@@ -64,7 +62,7 @@ class Postprocessing():
 
             # Save the figure and show
             plt.tight_layout()
-            plt.savefig(os.getcwd()+figuredirname+'realworld_benchmarks_bootstrapped_bar_plot_with_error_bars.png')
+            plt.savefig(os.path.join(self.figuredirname, 'realworld_benchmarks_bootstrapped_bar_plot_with_error_bars.png'))
             plt.show()
 
     def meta_simulation_visualise(self, results: list):
@@ -72,13 +70,15 @@ class Postprocessing():
         list_of_npractitioner_accuracies = results[1]
         list_of_nsl_accuracies = results[2]
         n_true_repetitions = results[3]
-        n_practitioner_repititions = results[4]
-        n_sl_repititions = results[5]
+        n_practitioner_repetitions = results[4]
+        n_sl_repetitions = results[5]
         self.dg_models = results[6]
         self.ml_models = results[7]
         list_of_ntrue_cv_accuracies = results[8]
         list_of_npractitioner_cv_accuracies = results[9]
         list_of_nsl_cv_accuracies = results[10]
+        list_of_dag_fidelity = results[11]
+        list_of_distribution_fidelity = results[12]
         ml_labels = [ml.name for ml in self.ml_models]
         sl_colors = {"hc": 'red', "tabu": 'blue', "rsmax2": 'green', "mmhc": 'yellow', "h2pc": 'orange', "gs": "cyan","pc.stable": "brown", "limited-real": "pink"}
         sl_offset = {"hc": 2, "tabu": 4, "rsmax2": 6, "mmhc": 8, "h2pc": 10, "gs": 12, "pc.stable": 14}
@@ -91,7 +91,7 @@ class Postprocessing():
         # Create boxplot for relative performance per pathways across all repetition for all ml methods
         output_pd_cv = pd.DataFrame(
             columns=['est', 'avgml', 'est-to-avgml', 'true', 'est-true', 'ML_method', 'SL', 'repetition'])
-        for practitioner_rep in range(n_practitioner_repititions):
+        for practitioner_rep in range(n_practitioner_repetitions):
             ml_method_cv_performance_list = []
 
             for ml in self.ml_models:
@@ -108,7 +108,7 @@ class Postprocessing():
                                             'ML_method': ml.name, 'SL': 'limited-real', 'repetition': practitioner_rep},
                                            ignore_index=True)
         for sl in self.dg_models:
-            for practitioner_rep in range(n_practitioner_repititions):
+            for practitioner_rep in range(n_practitioner_repetitions):
                 ml_method_cv_performance_list = []
 
                 for ml in self.ml_models:
@@ -123,15 +123,13 @@ class Postprocessing():
                                             'true': true_cv_performance,
                                             'est-true': sl_cv_performance - true_cv_performance, 'ML_method': ml.name,
                                             'SL': sl.SLClass, 'repetition': practitioner_rep}, ignore_index=True)
-        output_pd_cv.to_csv("meta_simulation_cv_performance_per_repetition.csv")
+        output_pd_cv.to_csv(os.path.join(self.figuredirname, "meta_simulation_cv_performance_per_repetition.csv"), index=False)
         output_pd_cv['ML_method'] = pd.Categorical(output_pd_cv['ML_method'], categories=ranked_cv_ml_methods, ordered=True)
 
         performance_diff_avgmean_table = output_pd_cv.pivot_table(index='SL', columns='ML_method', values='est-to-avgml',aggfunc='mean')
-        performance_diff_avgmean_table.to_csv("meta_simulation_cv_performance_difference_avgmean_table.csv")
-
+        performance_diff_avgmean_table.to_csv(os.path.join(self.figuredirname, "meta_simulation_cv_performance_difference_avgmean_table.csv"), index=False)
         performance_diff_true_table = output_pd_cv.pivot_table(index='SL', columns='ML_method', values='est-true',aggfunc='mean')
-        performance_diff_true_table.to_csv("meta_simulation_cv_performance_difference_true_table.csv")
-
+        performance_diff_true_table.to_csv(os.path.join(self.figuredirname, "meta_simulation_cv_performance_difference_true_table.csv"), index=False)
 
         plt.figure(figsize=(10, 10), dpi=300)
         sns.boxplot(x=output_pd_cv['ML_method'], y=output_pd_cv['est-true'], hue=output_pd_cv['SL'], palette=sl_colors,showfliers=True,showmeans=True,meanprops={"marker":"o","markerfacecolor":"black", "markeredgecolor":"grey","markersize":"5"},order=ranked_cv_ml_methods)
@@ -140,7 +138,7 @@ class Postprocessing():
         plt.xlabel('ML Method (least performing to best)')
         plt.xticks(rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_cv_diff_to_true_xaxis_ordered.png')
+        plt.savefig(os.path.join(self.figuredirname, "interworld_benchmarks_boxplot_all_ml_methods_cv_diff_to_true_xaxis_ordered.png"))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -150,7 +148,7 @@ class Postprocessing():
         plt.xlabel('ML Method (least performing to best)')
         plt.xticks(rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_cv_diff_to_avg_ml_performance.png')
+        plt.savefig(os.path.join(self.figuredirname,'interworld_benchmarks_boxplot_all_ml_methods_cv_diff_to_avg_ml_performance.png'))
         plt.show()
 
         # Scatterplot for cv
@@ -158,7 +156,7 @@ class Postprocessing():
         list_of_xy_pairs_cv = []
         dict_of_xy_pairs_sl_cv = {learner.SLClass: [] for learner in self.dg_models}
 
-        for practitioner_rep in range(n_practitioner_repititions):
+        for practitioner_rep in range(n_practitioner_repetitions):
             for ml in self.ml_models:
                 list_of_xy_pairs_cv.append((mean(list_of_ntrue_cv_accuracies[ml.name]),list_of_npractitioner_cv_accuracies[ml.name][practitioner_rep]))
                 for sl in dict_of_xy_pairs_sl_cv.keys():
@@ -197,11 +195,15 @@ class Postprocessing():
             plt.text(x=mean_cv_accuracies[ml_name], y=0.95, s=ml_name, fontsize=7, rotation='vertical', rotation_mode='anchor', horizontalalignment='center', verticalalignment='bottom', alpha=0.8)
         # Save and show the plot
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_scatterplot_every_practitioner_repetition_cv_accuracies_all_methods.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_scatterplot_every_practitioner_repetition_cv_accuracies_all_methods.png'))
         plt.show()
 
         scatter_cv_df = pd.DataFrame(scatter_cv_data, columns=['True_CV_Accuracy', 'Estimated_CV_Accuracy', 'Strategy'])
-        scatter_cv_df.to_csv(os.getcwd() + figuredirname + 'interworld_benchmarks_scatterplot_cv_true_est_accuracies_all_methods_table.csv', index=False)
+        scatter_cv_df.to_csv(
+            os.path.join(self.figuredirname,
+                         "interworld_benchmarks_scatterplot_cv_true_est_accuracies_all_methods_table.csv"),
+            index=False
+        )
 
         sl_differences_var_cv = pd.DataFrame(columns=['est-var', 'ML_method', 'SL'])
         for ml in self.ml_models:
@@ -210,10 +212,13 @@ class Postprocessing():
             for sl in self.dg_models:
                 sl_differences_var_cv = sl_differences_var_cv.append({'est-var': np.var(list_of_nsl_cv_accuracies[sl.SLClass][ml.name]), 'ML_method': ml.name, 'SL': sl.SLClass},ignore_index=True)
 
-        sl_differences_var_cv.to_csv("meta_simulation_cv_variance_per_ml_method.csv")
-
+        sl_differences_var_cv.to_csv(
+            os.path.join(os.getcwd(), figuredirname, "meta_simulation_cv_variance_per_ml_method.csv"),
+            index=False
+        )
         variance_table = sl_differences_var_cv.pivot(index='SL', columns='ML_method', values='est-var')
-        variance_table.to_csv("meta_simulation_cv_variance_table.csv")
+        variance_table.to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_cv_variance_table.csv"))
 
         plt.figure(figsize=(10, 10), dpi=300)
         sns.barplot(x=sl_differences_var_cv['ML_method'], y=sl_differences_var_cv['est-var'],hue=sl_differences_var_cv['SL'], data=sl_differences_var_cv)
@@ -222,7 +227,7 @@ class Postprocessing():
         plt.xlabel('ML Method')
         plt.xticks(rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_barplot_all_ml_methods_learner_cv_variance.png')
+        plt.savefig(os.path.join(self.figuredirname,'interworld_benchmarks_barplot_all_ml_methods_learner_cv_variance.png'))
         plt.show()
 
         pairwise_variance_test_results = []
@@ -262,12 +267,18 @@ class Postprocessing():
                 })
 
         # Exporting Results
-        pd.DataFrame(pairwise_variance_test_results).to_csv("meta_simulation_cv_pairwise_variance_test_results.csv", index=False)
-        pd.DataFrame(ci_results_var_diff).to_csv("meta_simulation_cv_ci_results_variance.csv", index=False)
+        pd.DataFrame(pairwise_variance_test_results).to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_cv_pairwise_variance_test_results.csv"),
+            index=False
+        )
 
+        pd.DataFrame(ci_results_var_diff).to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_cv_ci_results_variance.csv"),
+            index=False
+        )
         # Regular visualisations
         output_pd = pd.DataFrame(columns=['est','avgml','est-to-avgml','true','est-true', 'ML_method', 'SL','repetition'])
-        for practitioner_rep in range(0, n_practitioner_repititions):
+        for practitioner_rep in range(0, n_practitioner_repetitions):
             ml_method_performance_list = []
             #practitioner_rep_ml_methods = list_of_npractitioner_accuracies[practitioner_rep]
             for ml in self.ml_models:
@@ -282,7 +293,7 @@ class Postprocessing():
                                                     'ML_method': ml.name, 'SL': 'limited-real','repetition': practitioner_rep}, ignore_index=True)
 
         for sl in self.dg_models:
-            for practitioner_rep in range(0, n_practitioner_repititions):
+            for practitioner_rep in range(0, n_practitioner_repetitions):
                 ml_method_performance_list = []
                 for ml in self.ml_models:
                     ml_method_performance_list.append(list_of_nsl_accuracies[sl.SLClass][ml.name][practitioner_rep])
@@ -291,31 +302,34 @@ class Postprocessing():
                     sl_rep_ml_performance = list_of_nsl_accuracies[sl.SLClass][ml.name][practitioner_rep]
                     true_rep_ml_performance = mean(list_of_ntrue_accuracies[ml.name])
                     output_pd = output_pd.append({'est': sl_rep_ml_performance,'avgml': avg_ml_rep_performance, 'est-to-avgml': (sl_rep_ml_performance - avg_ml_rep_performance),'true': true_rep_ml_performance, 'est-true': (sl_rep_ml_performance - true_rep_ml_performance),'ML_method': ml.name, 'SL': sl.SLClass, 'repetition': practitioner_rep}, ignore_index=True)
-        output_pd.to_csv("meta_simulation_performance_per_repetition.csv")
+        output_pd.to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_performance_per_repetition.csv"),
+            index=False  # optional, add if you don't want the index
+        )
 
         # Create absolute match proportion bar graph between pathway's top ml method and true ml method
         matches_from_practitioner_limited_world = 0
         sl_match_counts = {learner.SLClass: 0 for learner in self.dg_models}
-        for repetition in range(0, n_practitioner_repititions):
-            list_of_comparable_ranks_in_one_repitition = {method.name: 0 for method in self.ml_models}
-            list_of_comparable_ranks_in_one_repitition_b = {method.name: 0 for method in self.ml_models}
+        for repetition in range(0, n_practitioner_repetitions):
+            list_of_comparable_ranks_in_one_repetition = {method.name: 0 for method in self.ml_models}
+            list_of_comparable_ranks_in_one_repetition_b = {method.name: 0 for method in self.ml_models}
             for ml in self.ml_models:
-                list_of_comparable_ranks_in_one_repitition[ml.name] = mean(list_of_ntrue_accuracies[ml.name])#[repetition]
-                list_of_comparable_ranks_in_one_repitition_b[ml.name] = list_of_npractitioner_accuracies[ml.name][repetition]
-            if max(list_of_comparable_ranks_in_one_repitition_b,key=list_of_comparable_ranks_in_one_repitition_b.get) == max(list_of_comparable_ranks_in_one_repitition, key=list_of_comparable_ranks_in_one_repitition.get):
+                list_of_comparable_ranks_in_one_repetition[ml.name] = mean(list_of_ntrue_accuracies[ml.name])#[repetition]
+                list_of_comparable_ranks_in_one_repetition_b[ml.name] = list_of_npractitioner_accuracies[ml.name][repetition]
+            if max(list_of_comparable_ranks_in_one_repetition_b,key=list_of_comparable_ranks_in_one_repetition_b.get) == max(list_of_comparable_ranks_in_one_repetition, key=list_of_comparable_ranks_in_one_repetition.get):
                 matches_from_practitioner_limited_world += 1
         for sl in sl_match_counts.keys():
             sl_match_count = 0
-            for repetition in range(0, n_practitioner_repititions):
-                list_of_comparable_ranks_in_one_repitition = {method.name: 0 for method in self.ml_models}
-                list_of_comparable_ranks_in_one_repitition_b = {method.name: 0 for method in self.ml_models}
+            for repetition in range(0, n_practitioner_repetitions):
+                list_of_comparable_ranks_in_one_repetition = {method.name: 0 for method in self.ml_models}
+                list_of_comparable_ranks_in_one_repetition_b = {method.name: 0 for method in self.ml_models}
                 for ml in self.ml_models:
-                    list_of_comparable_ranks_in_one_repitition[ml.name] = mean(list_of_ntrue_accuracies[ml.name])#[repetition]
-                    list_of_comparable_ranks_in_one_repitition_b[ml.name] = list_of_nsl_accuracies[sl][ml.name][repetition]
-                if max(list_of_comparable_ranks_in_one_repitition_b,key=list_of_comparable_ranks_in_one_repitition_b.get) == max(list_of_comparable_ranks_in_one_repitition, key=list_of_comparable_ranks_in_one_repitition.get):
+                    list_of_comparable_ranks_in_one_repetition[ml.name] = mean(list_of_ntrue_accuracies[ml.name])#[repetition]
+                    list_of_comparable_ranks_in_one_repetition_b[ml.name] = list_of_nsl_accuracies[sl][ml.name][repetition]
+                if max(list_of_comparable_ranks_in_one_repetition_b,key=list_of_comparable_ranks_in_one_repetition_b.get) == max(list_of_comparable_ranks_in_one_repetition, key=list_of_comparable_ranks_in_one_repetition.get):
                     sl_match_count += 1
-            sl_match_counts[sl] = sl_match_count / n_practitioner_repititions
-        sl_match_counts["limited-real"] = matches_from_practitioner_limited_world / n_practitioner_repititions
+            sl_match_counts[sl] = sl_match_count / n_practitioner_repetitions
+        sl_match_counts["limited-real"] = matches_from_practitioner_limited_world / n_practitioner_repetitions
         fig = plt.figure(figsize=(10, 10), dpi=300)
         plt.bar(list(sl_match_counts.keys()), list(sl_match_counts.values()), color='maroon', width=0.4)
         plt.xlabel("Technique")
@@ -323,7 +337,9 @@ class Postprocessing():
         plt.title("Proportion of matched top ML methods from technique to true top benchmarks")
         plt.style.use("seaborn")
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_barplot_proportion_matches_max_method.png')
+        plt.savefig(
+            os.path.join(self.figuredirname, "interworld_benchmarks_barplot_proportion_matches_max_method.png")
+        )
         plt.show()
 
         # Create scatterplot of relative performances for every repetition for all ml methods between alternative pathways
@@ -332,7 +348,7 @@ class Postprocessing():
         list_of_xy_pairs_all_methods_limited_world = []
         dict_of_xy_pairs_sl = {learner.SLClass: [] for learner in self.dg_models}
 
-        for repetition in range(0, n_practitioner_repititions):
+        for repetition in range(0, n_practitioner_repetitions):
             for ml_index, ml_method_label in enumerate(self.ml_models):
                 list_of_xy_pairs_all_methods_limited_world.append((mean(list_of_ntrue_accuracies[ml_method_label.name]),list_of_npractitioner_accuracies[ml_method_label.name][repetition]))
                 for sl in dict_of_xy_pairs_sl.keys():
@@ -373,19 +389,21 @@ class Postprocessing():
         for ml_index, ml_method_label in enumerate(self.ml_models):
             plt.text(x=x_true_accuracies[ml_index], y=0.95, s=ml_method_label.name,fontsize=7, rotation='vertical',rotation_mode='anchor',horizontalalignment='center', verticalalignment='bottom', alpha=0.8)#y=y_limited_accuracies[ml_index]
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_scatterplot_every_practitioner_repetition_accuracies_all_methods.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_scatterplot_every_practitioner_repetition_accuracies_all_methods.png'))
         plt.show()
 
         scatter_df = pd.DataFrame(scatter_data, columns=['True_Accuracy','Estimated_Accuracy', 'Strategy'])
-        scatter_df.to_csv(os.getcwd()+figuredirname+ 'interworld_benchmarks_scatterplot_true_est_accuracies_all_methods_table.csv', index=False)
-
+        scatter_df.to_csv(
+            os.path.join(self.figuredirname, 'interworld_benchmarks_scatterplot_true_est_accuracies_all_methods_table.csv'),
+            index=False
+        )
         # Barplot calculation of std
         sl_differences_var = pd.DataFrame(columns=['est-var', 'ML_method', 'SL'])
         sl_differences_std = pd.DataFrame(columns=['est-std', 'ML_method', 'SL'])
 
         #practitioner_ml_method_performance_list = {method.name: [] for method in self.ml_models}
         for ml in self.ml_models:
-            #for practitioner_rep in range(0, n_practitioner_repititions):
+            #for practitioner_rep in range(0, n_practitioner_repetitions):
             #    practitioner_ml_method_performance_list[ml.name].append(list_of_npractitioner_accuracies[practitioner_rep][ml.name])
             sl_differences_var = sl_differences_var.append({'est-var': np.var(list_of_npractitioner_accuracies[ml.name]),'ML_method': ml.name,'SL': 'limited-real'}, ignore_index=True)
             sl_differences_std = sl_differences_std.append({'est-std': np.std(list_of_npractitioner_accuracies[ml.name]), 'ML_method': ml.name, 'SL': 'limited-real'},ignore_index=True)
@@ -393,7 +411,7 @@ class Postprocessing():
         for ml in self.ml_models:
             for sl in self.dg_models:
                 #sl_ml_method_performance_list = {method.name: [] for method in self.ml_models}
-                #for practitioner_rep in range(0, n_practitioner_repititions):
+                #for practitioner_rep in range(0, n_practitioner_repetitions):
                 #    sl_ml_method_performance_list[ml.name].append(list_of_nsl_accuracies[sl.SLClass][ml.name][practitioner_rep])
                 sl_differences_var = sl_differences_var.append({'est-var': np.var(list_of_nsl_accuracies[sl.SLClass][ml.name]),'ML_method': ml.name,'SL': sl.SLClass}, ignore_index=True)
                 sl_differences_std = sl_differences_std.append({'est-std': np.std(list_of_nsl_accuracies[sl.SLClass][ml.name]),'ML_method': ml.name, 'SL': sl.SLClass}, ignore_index=True)
@@ -405,7 +423,7 @@ class Postprocessing():
         plt.xlabel('ML Method')
         ax.tick_params(labelrotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_barplot_all_ml_methods_learner_var.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_barplot_all_ml_methods_learner_var.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -415,11 +433,13 @@ class Postprocessing():
         plt.xlabel('ML Method')
         ax.tick_params(labelrotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_barplot_all_ml_methods_learner_std.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_barplot_all_ml_methods_learner_std.png'))
         plt.show()
 
-        sl_differences_var.to_csv("meta_simulation_variance_per_ml_method.csv")
-
+        sl_differences_var.to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_variance_per_ml_method.csv"),
+            index=False  # optional, add if you don't want the index
+        )
         pairwise_variance_test_results = []
         ci_results_var_diff = []
 
@@ -459,9 +479,15 @@ class Postprocessing():
                 })
 
         # Exporting Results
-        pd.DataFrame(pairwise_variance_test_results).to_csv("meta_simulation_pairwise_variance_test_results.csv", index=False)
-        pd.DataFrame(ci_results_var_diff).to_csv("meta_simulation_ci_results_variance.csv", index=False)
+        pd.DataFrame(pairwise_variance_test_results).to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_pairwise_variance_test_results.csv"),
+            index=False
+        )
 
+        pd.DataFrame(ci_results_var_diff).to_csv(
+            os.path.join(self.figuredirname, "meta_simulation_ci_results_variance.csv"),
+            index=False
+        )
         plt.figure(figsize=(10, 10), dpi=300)
         ax = sns.boxplot(x=output_pd['ML_method'], y=output_pd['est-true'], hue=output_pd['SL'],palette=sl_colors,showfliers=False, showmeans=True,meanprops={"marker":"o","markerfacecolor":"black", "markeredgecolor":"grey","markersize":"5"})
         plt.title('Boxplot of difference between estimated performances and true by ml method for strategies')
@@ -469,7 +495,7 @@ class Postprocessing():
         plt.xlabel('ML Method')
         ax.set_xticklabels(ml_labels, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -483,7 +509,7 @@ class Postprocessing():
             ax.get_xticklabels()
         ax.set_xticklabels(added_methods, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true_xaxis_ordered.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true_xaxis_ordered.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -497,7 +523,7 @@ class Postprocessing():
             ax.get_xticklabels()
         ax.set_xticklabels(added_methods, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true_xaxis_ordered_fliers.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_true_xaxis_ordered_fliers.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -507,7 +533,7 @@ class Postprocessing():
         plt.xlabel('ML Method')
         ax.set_xticklabels(ml_labels, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -521,7 +547,7 @@ class Postprocessing():
             ax.get_xticklabels()
         ax.set_xticklabels(added_methods, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance_xaxis_ordered.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance_xaxis_ordered.png'))
         plt.show()
 
         plt.figure(figsize=(10, 10), dpi=300)
@@ -535,7 +561,7 @@ class Postprocessing():
             ax.get_xticklabels()
         ax.set_xticklabels(added_methods, rotation=90)
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance_xaxis_ordered_fliers.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_boxplot_all_ml_methods_diff_to_avg_ml_performance_xaxis_ordered_fliers.png'))
         plt.show()
 
         for ml in self.ml_models:
@@ -547,7 +573,7 @@ class Postprocessing():
             plt.ylabel("Density")
             #plt.legend()#ml_method_filtered_performance['SL'])
             plt.tight_layout()
-            plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_kdeplot_performance_'+ml.name+'by_sl.png')
+            plt.savefig(os.path.join(self.figuredirname, f'interworld_benchmarks_kdeplot_performance_{ml.name}by_sl.png'))
             plt.show()
 
         violin_header = [method.name for method in self.ml_models]
@@ -556,7 +582,7 @@ class Postprocessing():
         csv_performances = pd.DataFrame(columns=violin_header)
         ranked_csv_performances = pd.DataFrame(columns=violin_header)
         violinplot_performances = pd.DataFrame({'ML_Method': pd.Series(dtype='str'),'rank': pd.Series(dtype='int'),'SL': pd.Series(dtype='str')})
-        for practitioner_rep in range(0, n_practitioner_repititions):
+        for practitioner_rep in range(0, n_practitioner_repetitions):
             practitioner_rep_ml_methods = {method.name: list_of_npractitioner_accuracies[method.name][practitioner_rep] for method in self.ml_models}#list_of_npractitioner_accuracies[practitioner_rep]
             csv_performances = csv_performances.append({method.name: practitioner_rep_ml_methods[method.name] for method in self.ml_models}, ignore_index=True)
             ranked_performances = pd.DataFrame.from_dict([dict(zip(practitioner_rep_ml_methods.keys(), rankdata([-i for i in practitioner_rep_ml_methods.values()], method='min')))], dtype=int64)
@@ -567,7 +593,7 @@ class Postprocessing():
         for sl in self.dg_models:
             csv_performances = pd.DataFrame(columns=self.ml_models)
             ranked_csv_performances = pd.DataFrame(columns=self.ml_models)
-            for sl_rep in range(0, n_practitioner_repititions):
+            for sl_rep in range(0, n_practitioner_repetitions):
                 sl_accuracies = {method.name: 0 for method in self.ml_models}
                 for ml in self.ml_models:
                     sl_accuracies[ml.name] = list_of_nsl_accuracies[sl.SLClass][ml.name][sl_rep]
@@ -587,13 +613,48 @@ class Postprocessing():
         ax.set_xticklabels(ordered_list_of_methods, rotation=90)
         plt.xlabel("ML Methods (least performing to best)")
         plt.tight_layout()
-        plt.savefig(os.getcwd() + figuredirname + 'interworld_benchmarks_violinplot_every_repetition_rank_all_methods_composite.png')
+        plt.savefig(os.path.join(self.figuredirname, 'interworld_benchmarks_violinplot_every_repetition_rank_all_methods_composite.png'))
         plt.show()
 
-    def dict_to_list(self, score_name, all_results: dict):
+        dag_fid_df = pd.concat(
+            {sl: pd.DataFrame(list_of_dag_fidelity[sl]) for sl in list_of_dag_fidelity},
+            names=["SL", "rep"]
+        ).reset_index(level="SL")
+
+        plt.figure(figsize=(8, 6))
+        sns.boxplot(x="SL", y="SHD", data=dag_fid_df, palette=sl_colors)
+        plt.title("Structural Hamming Distance (SHD) of Learned DAGs")
+        plt.ylabel("SHD (lower is better)")
+        plt.xticks(rotation=45)
+        plt.tight_layout()
+        plt.savefig(os.path.join(self.figuredirname, "dag_fidelity_shd_boxplot.png"))
+        plt.show()
+
+        dist_fid_df = pd.concat(
+            {sl: pd.DataFrame(list_of_distribution_fidelity[sl]) for sl in list_of_distribution_fidelity},
+            names=["SL", "rep"]
+        ).reset_index(level="SL")
+
+        plt.figure(figsize=(8, 6))
+        sns.barplot(x="SL", y="JS_divergence", data=dist_fid_df, palette=sl_colors, ci="sd")
+        plt.title("Distribution Fidelity (JS divergence)")
+        plt.ylabel("JS divergence (lower is better)")
+        plt.xticks(rotation=45)
+        plt.tight_layout()
+        plt.savefig(os.path.join(self.figuredirname, "distribution_fidelity_barplot.png"))
+        plt.show()
+
+    def dict_to_list(self, score_name: str, all_results: dict):
+        """
+        Convert results dict to list-of-lists table.
+        all_results: {world_name: {ml_model: {score_name: value}}}
+        """
         data = []
-        ml_algs = list(TD(all_results, 1).keys())
+        # get ml_algs from the first world entry
+        first_world = next(iter(all_results.values()))
+        ml_algs = list(first_world.keys())
         worlds = list(all_results.keys())
+
         for alg in ml_algs:
             inner_list = [alg]
             for world in worlds:
@@ -603,14 +664,24 @@ class Postprocessing():
 
     # not used for now
     def get_true_performance_stats(self, scores: dict):
-        # scores: dict of shape {"ml_model_name": {"score_name": list_of_values}}
-        score_names = TD(scores, 1).keys()
-        stats = {ml_model_name: {score_name: {} for score_name in score_names} for ml_model_name in scores.keys()}
-        for ml_model_name in scores.keys():
-            for score in score_names:
-                stats[ml_model_name][score]["min"] = np.min(scores[ml_model_name][score])
-                stats[ml_model_name][score]["max"] = np.max(scores[ml_model_name][score])
-                stats[ml_model_name][score]["mean"] = np.mean(scores[ml_model_name][score])
+        """
+        scores: {ml_model: {score_name: list_of_values}}
+        """
+        # gather score names from first model
+        first_model = next(iter(scores.values()))
+        score_names = list(first_model.keys())
+
+        stats = {
+            ml_model: {
+                score_name: {
+                    "min": np.min(scores[ml_model][score_name]),
+                    "max": np.max(scores[ml_model][score_name]),
+                    "mean": np.mean(scores[ml_model][score_name]),
+                }
+                for score_name in score_names
+            }
+            for ml_model in scores.keys()
+        }
         return stats
 
     def plot_analysis_coef(self, analysis2_results: dict, pipeline: str = "Real-world"):
@@ -631,24 +702,21 @@ class Postprocessing():
         plt.ylabel("learned world correlations")
         plt.legend()
         plt.tight_layout()
-        plt.savefig('scatter_plot_of_ccs.png')
+        plt.savefig(os.path.join(os.getcwd(), figuredirname, 'scatter_plot_of_ccs.png'))
         plt.show()
 
-    def dict_to_dataframe_sns(self, scores: dict, score_name: str = 'balanced_accuracy_score') -> pd.DataFrame:
-        '''
-
-        :param scores: dict of form {world_name: {ml_model_name: {score_name: list of scores]...}...}...}
-        :param score_name:
-        :return: dataframe with columns=[world, ml_model, score] and rows corresponding to individual entries
-        '''
-        score_dict = TD(scores, 2)[score_name]
-        raw_scores_df = pd.DataFrame(score_dict)
+    def dict_to_dataframe_sns(self, scores: dict, score_name: str = "balanced_accuracy_score") -> pd.DataFrame:
+        """
+        scores: {world: {ml_model: {score_name: list_of_scores}}}
+        Returns: pd.DataFrame with columns [world, ml_model, score]
+        """
         list_of_lists = []
-
-        for col in raw_scores_df.columns:
-            for row in raw_scores_df.index:
-                for i in range(5):
-                    list_of_lists.append([col, row, raw_scores_df[col][row][i]])
+        for world, models in scores.items():
+            for ml_model, score_dict in models.items():
+                if score_name not in score_dict:
+                    continue
+                for value in score_dict[score_name]:
+                    list_of_lists.append([world, ml_model, value])
 
         return pd.DataFrame(list_of_lists, columns=["world", "ml_model", "score"])
 
