@@ -1,6 +1,6 @@
 import numpy as np
 from dagsim.base import Graph, Node
-from src.simcalibration.dg_models.DagsimModel import DagsimModel
+from simcalibration.dg_models.DagsimModel import DagsimModel
 
 def get_asia():
     if np.random.binomial(n=1, p=0.01):
@@ -1610,6 +1610,7 @@ def get_printer():
                  Prior57, Prior58, Prior59, Prior60, Prior61, Prior62, Prior63, Prior64, Prior65, Prior66, Prior67,
                  Prior68, Prior69, Prior70, Prior71, Prior72, Prior73, Prior74, Prior75, Prior76]
     printer = Graph(name="Windows 95 Printer Example - Real-world", list_nodes=listNodes)
+    # printer.draw()
     ds_model = DagsimModel("Real-world", printer)
     return ds_model
 
@@ -1626,6 +1627,7 @@ def get_asia():
 
     listNodes = [Prior1, Prior2, Prior3, Prior4, Prior5, Prior6, Prior7, Prior8]
     asia = Graph(name="Asia Cancer Dysponea Example - Real-world", list_nodes=listNodes)
+    asia.draw()
     ds_model = DagsimModel("Real-world", asia)
     return ds_model
 
